/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */
#include "shopper.h"           // automake, i8n, gettext
#include <sstream>
#include <gtkmm.h>
#include <hildon-fmmm.h>
#include <glibmm.h>
#include "CategoryDialog.h"

//#define DEBUG(...) cerr << __VA_ARGS__
#define DEBUG(...)

namespace Shopper
{
////////////////////////////////////////////////////////////////
// Public
	CategoryManage::CategoryManage(Shopper::List* l) :
		CategoryDialog(l, NULL)
	{}

	int CategoryDialog::run()
	{
		return Gtk::Dialog::run();
	}

////////////////////////////////////////////////////////////////
// Private
	CategoryDialog::CategoryDialog(Shopper::List* l, Shopper::Category* cat) :
		Gtk::Dialog("Manage Categories", true, true),
		mylist(l),
		mycat(cat),
		vb(NULL)
	{
		// Display the buttons
		Gtk::Button *add_b = manage(new Gtk::Button("Add Category"));
		add_b->signal_clicked()
			.connect(sigc::mem_fun(*this, &Shopper::CategoryDialog::add_category));
		get_action_area()->add(*add_b);
		add_b->show();
		add_button("Done", Gtk::RESPONSE_ACCEPT);

		Gtk::VBox *v = 	get_vbox();
		v->add(sw);
		update(); // add the current list.
		// Now connect the List's state changed signal to this object's updater
		l->watch().connect( sigc::mem_fun(*this, &Shopper::CategoryDialog::list_changed) );

		// geometric aesthetics
		int h=get_screen()->get_height();
		int w=get_screen()->get_width();
		resize((w*6)/10,h);
		show();
	}

	void CategoryDialog::list_changed(List_action action, List *l, Category *c, Item *i)
	{
		switch (action){
		case LIST_STATE_CHANGED:
		case CATEGORY_LIST_CHANGED:
		case ITEM_LIST_CHANGED:
		case LIST_ACTIVE_CAT_CHANGED:
			update();
			break;
		default:
			DEBUG("list_changed: Unknown action : "
				  << Shopper::watchable::action_name(action) << "\n");
			break;
		}
	}

	void CategoryDialog::update()
	{
		delete vb; // This should delete everything cleanly...

		// start again - hmm - what happens to our scrolled view?
		vb = new Gtk::VBox;
		// lots of these
		CatView *cv;
		Shopper::List::pCategoryIter begin =mylist->categoriesI(),
			end = mylist->categoriesEnd();
		for(Shopper::List::pCategoryIter catI = begin; catI != end; ++catI)
		{
			cv = manage(new CatView(this, **catI, *mylist));  // vb->manage means deletion with vb.
			vb->pack_start(*cv, Gtk::PACK_SHRINK);   // add CatView to the scrolled vb
			if (catI == begin) cv->set_top(); 
			cv->show();
		}
		cv->set_bottom();
		vb->show();
		sw.add(*vb);
		sw.show();
	}
	
	void CategoryDialog::add_category()
	{
		// Create a new category
		mycat = new Shopper::Category("New Category");
		mylist->add(*mycat);
		// FIXME: set the sw->scroll and category edit focus here
	}


////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////

// The widget shows the state of a category
// it can move up or down and edits the relevant category.
	CatView::CatView(CategoryDialog *d, Category &cat, List &lst) :
		myD(d),
		mycat(&cat),
		mylist(&lst),
		name_l(cat.get_name(), Gtk::ALIGN_LEFT),
		name_edit(NULL),
		b_del(),
		b_up(),
		at_top(false),
		b_down(),
		at_bottom(false),
		i_del(Gtk::Stock::DELETE, Gtk::ICON_SIZE_SMALL_TOOLBAR),
		i_up(Gtk::Stock::GO_UP, Gtk::ICON_SIZE_SMALL_TOOLBAR),
		i_down(Gtk::Stock::GO_DOWN, Gtk::ICON_SIZE_SMALL_TOOLBAR)
	{
		// Static images used by all buttons
		// FIXME : see if we can make these static - if they're not.
		b_del.add(i_del);
		b_up.add(i_up);
		b_down.add(i_down);
		Shopper::List::pCategoryIter
			begin =mylist->categoriesI(),
			end = mylist->categoriesEnd();
		name_b.add(name_l); // label in event box
		pack_start(b_del, Gtk::PACK_SHRINK);  // del button
		pack_start(name_b, Gtk::PACK_EXPAND_WIDGET);  // ev box in hbox, expanded
		pack_start(b_up, Gtk::PACK_SHRINK);         // add up + down buttons
		pack_start(b_down, Gtk::PACK_SHRINK);
		// visibility:
		show_all();

		// add all the callbacks
		// FIXME

		// Respond to clicking the label
		name_b.signal_button_press_event()
			.connect( sigc::mem_fun(*this, &Shopper::CatView::name_show_edit) );
		
		// Respond to del
		b_del.signal_clicked()
			.connect( sigc::mem_fun(*this, &Shopper::CatView::delete_category) );
		// Respond to up
		b_up.signal_clicked()
			.connect( sigc::mem_fun(*this, &Shopper::CatView::move_up) );
		// Respond to down
		b_down.signal_clicked()
			.connect( sigc::mem_fun(*this, &Shopper::CatView::move_down) );
		
		// Make the container visible according to visibility logic
		DEBUG("CatView created for" <<cat.get_name());
	}

	CatView::~CatView()
	{
		delete name_edit;
		name_edit=NULL;
	}

	void CatView::delete_category()
	{
		DEBUG("Delete Category\n");
		if (mylist->is_category_active(*mycat)) {
			Hildon::Banner::show_information(*this, "Can't delete current category");
			return;
		}
		
		Gtk::Dialog dia("Are you sure?", true, true);
		std::ostringstream size;
		int s = mycat->get_size();
		size << s;
		Gtk::Label *l;
		if (s>0) {
			l = Gtk::manage(new Gtk::Label("Category '"
										   + mycat->get_name()
										   + "' has " + size.str()
										   + " items. Delete them all? "));
		} else {
			l = manage(new Gtk::Label("Delete empty category '"
									  + mycat->get_name() + "'?"));
		}
		dia.get_vbox()->add(*l);
		l->show();
		dia.add_button(Gtk::Stock::YES, Gtk::RESPONSE_ACCEPT);
		dia.add_button(Gtk::Stock::NO, Gtk::RESPONSE_REJECT);
		if (dia.run() == Gtk::RESPONSE_ACCEPT)
		{
			Hildon::Banner::show_information(*this, "Deleted category " + mycat->get_name());
			mylist->rm(*mycat);
		}
	}

// Only shows if matching item should be seen.
	void CatView::show()
	{
		// FIXME - determine position and hide buttons if needed.
		DEBUG(" show\n");
		name_l.show();
		name_b.show();
		b_up.show();
		b_down.show();
		if (at_top) b_up.hide();
		if (at_bottom) b_down.hide();
		Gtk::HBox::show();
	}

	bool CatView::name_show_edit(GdkEventButton* ev)
	{
		if (ev->type != GDK_BUTTON_PRESS) return false;
		if (name_edit != NULL) return true; // handle double clicks
		DEBUG("Pressed button " << ev->button << "\n");
		name_edit = new Gtk::Entry();
		name_edit->set_activates_default(false);
		name_edit->set_width_chars(15); // FIXME
		name_edit->set_text(mycat->get_name());
		// self-destruct on unfocus or activate
		name_edit->signal_activate()  // FIXME : Crashes. activate conflicts withe name_b.remove()
			.connect( sigc::mem_fun(*this,
									(void (Shopper::CatView::*)())
									&Shopper::CatView::name_show_label) );
		name_edit->signal_button_press_event()
			.connect( sigc::mem_fun(*this,
									(bool (Shopper::CatView::*)(GdkEventButton*))
									&Shopper::CatView::name_show_label) );
		name_edit->signal_focus_out_event()
			.connect( sigc::mem_fun(*this,
									(bool (Shopper::CatView::*)(GdkEventFocus*))
									&Shopper::CatView::name_show_label) );
		
		name_b.remove(); // remove label
		name_b.add(*name_edit);
		name_edit->show();
		name_edit->grab_focus();
		return true;
	}
	bool CatView::name_show_label(GdkEventFocus* ev)
	{
		if (! ev->in){
			name_show_label();
			return true; // handled
		}
		return false; // not handled
	}
	bool CatView::name_show_label(GdkEventButton* ev)
	{
		if (ev->type != GDK_BUTTON_PRESS) return false;
		if (ev->button ==3 ){
			DEBUG("Middle click\n");
			name_show_label();
			return true ; //handled
		}
		return false; // not handled
	}
	void CatView::name_show_label()
	{
		if (name_edit == NULL) return; // handle queued events...
		mycat->set_name(name_edit->get_text()); 
		delete name_edit;
		name_edit=NULL;
		name_b.remove(); // remove name_edit
		name_b.add(name_l);
		name_l.show();
		name_l.set_text(mycat->get_name()); // Label has no callback
	}
	void CatView::set_top()
	{
			//FIXME - hide button
		at_top = true;
		show();
	}
	void CatView::set_bottom()
	{
		//FIXME - hide button
		at_bottom = true;
		show();
	}
	void CatView::move_up()
	{
		Gtk::Box_Helpers::BoxList::iterator me = myD->vb->children().find(*this);
		if (me == myD->vb->children().begin()) {
			return;
		}
		me--;
		CatView *that = static_cast<CatView*>(me->get_widget());

		mylist->swap_categories(mycat, that->mycat);
		if (at_bottom) {
			at_bottom=false;
			that->at_bottom=true;
		}
		that->at_top=false;
		if (me == myD->vb->children().begin()) {
			at_top=true;
		}		
		show();
		that->show();
		myD->vb->reorder_child(*this, mycat->get_id());
	}
	void CatView::move_down()
	{
		Gtk::Box_Helpers::BoxList::iterator me = myD->vb->children().find(*this);
		me++;
		if (me == myD->vb->children().end()) {
			return;
		}
		CatView *that = static_cast<CatView*>(me->get_widget());
		mylist->swap_categories(mycat, that->mycat);
		if (at_top) {
			at_top=false;
			that->at_top=true;
		}
		that->at_bottom=false;
		if (++me == myD->vb->children().end()) {
			DEBUG("at bottom now\n");
			at_bottom=true;
		}
		show();
		that->show();
		myD->vb->reorder_child(*this, mycat->get_id());
	}
}
